'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _SWebComponent2 = require('coffeekraken-sugar/js/core/SWebComponent');

var _SWebComponent3 = _interopRequireDefault(_SWebComponent2);

var _getTransitionProperties = require('coffeekraken-sugar/js/dom/getTransitionProperties');

var _getTransitionProperties2 = _interopRequireDefault(_getTransitionProperties);

var _scrollTop = require('coffeekraken-sugar/js/dom/scrollTop');

var _scrollTop2 = _interopRequireDefault(_scrollTop);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @name 		SDrawerComponent
 * @extends 	SWebComponent
 * Simple webcomponent to create fully customizable drawers.
 * Features:
 * 1. Fully customizable
 * 2. Support all sides (top, right, bottom and left)
 * 3. Support 3 animation types (push, slide and reveal)
 *
 * @example 	scss
 * \@include s-drawer-classes(
 * 	$name : menu,
 * 	$side : right
 * );
 * \@include s-drawer-element(drawer) {
 * 	background-color: black;
 * 	padding: 20px;
 * }
 * \@include s-drawer-element(overlay) {
 * 	background-color: rgba(0,0,0,.5);
 * }
 *
 * @author 		Olivier Bossel <olivier.bossel@gmail.com>
 */

var SDrawerComponent = function (_SWebComponent) {
	_inherits(SDrawerComponent, _SWebComponent);

	function SDrawerComponent() {
		_classCallCheck(this, SDrawerComponent);

		return _possibleConstructorReturn(this, (SDrawerComponent.__proto__ || Object.getPrototypeOf(SDrawerComponent)).apply(this, arguments));
	}

	_createClass(SDrawerComponent, [{
		key: 'componentMount',


		/**
   * Mount component
   * @definition 		SWebComponent.componentMount
   * @protected
   */
		value: function componentMount() {
			var _this2 = this;

			_get(SDrawerComponent.prototype.__proto__ || Object.getPrototypeOf(SDrawerComponent.prototype), 'componentMount', this).call(this);

			this.contentElm = document.querySelector('[' + this.componentNameDash + '-content]');

			// try to find the drawer background
			this.bkgElm = document.querySelector('[' + this.componentNameDash + '-bkg][for="' + this.props.name + '"]');
			if (!this.bkgElm) {
				this.bkgElm = document.createElement('div');
				this.mutate(function () {
					_this2.bkgElm.setAttribute(_this2.componentNameDash + '-bkg', true);
					_this2.bkgElm.setAttribute('for', _this2.props.name);
					// insert in the page
					_this2.parentElement.insertBefore(_this2.bkgElm, _this2.parentElement.firstChild);
				});
			}

			// try to find the drawer overlay
			this.overlayElm = document.querySelector('label[is="' + this.componentNameDash + '-overlay"][for="' + this.props.name + '"]');
			if (!this.overlayElm) {
				this.overlayElm = document.createElement('label');
				this.overlayElm.setAttribute('for', this.props.name);
				this.overlayElm.setAttribute(this.componentNameDash + '-overlay', true);
				this.mutate(function () {
					// insert in the page
					_this2.parentElement.insertBefore(_this2.overlayElm, _this2.parentElement.firstChild);
				});
			}

			// try to find the toggle
			this.toggleElm = document.querySelector('input[is="' + this.componentNameDash + '-toggle"][name="' + this.props.name + '"]');
			if (!this.toggleElm) {
				this.toggleElm = document.createElement('input');
				this.toggleElm.setAttribute('name', this.props.name);
				this.toggleElm.setAttribute('id', this.props.name);
				this.toggleElm.setAttribute('type', 'checkbox');
				this.toggleElm.setAttribute(this.componentNameDash + '-toggle', true);
				this.mutate(function () {
					// insert into page
					_this2.parentElement.insertBefore(_this2.toggleElm, _this2.parentElement.firstChild);
				});
			}

			this._scrollTop = 0;

			// listen for change on the toggle
			this.toggleElm.addEventListener('change', function (e) {
				var name = e.target.name;
				_this2.mutate(function () {
					if (e.target.checked) {
						document.body.classList.add(_this2.componentNameDash + '-' + _this2.props.name);
						if (_this2.props.preventContentScroll) {
							_this2._scrollTop = (0, _scrollTop2.default)();
							_this2.contentElm.style.transition = 'none';
							_this2.mutate(function () {
								_this2.contentElm.style.transform = 'translateY(-' + _this2._scrollTop + 'px)';
								_this2.ownerDocument.body.style.position = 'fixed';
								_this2.ownerDocument.body.style.overflow = 'hidden';
								setTimeout(function () {
									_this2.contentElm.style.transition = '';
								});
							});
						}
					} else {
						document.body.classList.remove(_this2.componentNameDash + '-' + _this2.props.name);
						if (_this2.props.preventContentScroll) {
							_this2.contentElm.style.transition = 'none';
							_this2.mutate(function () {
								_this2.contentElm.style.transform = '';
								_this2.ownerDocument.body.style.position = '';
								_this2.ownerDocument.body.style.overflow = '';
								window.scrollTo(0, _this2._scrollTop);
								setTimeout(function () {
									_this2.contentElm.style.transition = '';
								});
							});
						}
					}
				});
			});

			// listen for click on links into the drawer to close it
			if (this.props.closeOnClick) {
				this.addEventListener('click', function (e) {
					if (e.target.tagName.toLowerCase() === 'a') {
						// close the drawer
						_this2.close();
					}
				});
			}

			// if handle hach
			if (this.props.handleHash) {
				if (document.location.hash) {
					var hash = document.location.hash.substr(1);
					if (hash == this.props.name) {
						this.open();
					}
				}
			}
		}

		/**
   * Open the drawer
   */

	}, {
		key: 'open',
		value: function open() {
			var _this3 = this;

			// check the toggle
			this.mutate(function () {
				_this3.toggleElm.setAttribute('checked', true);
				document.body.classList.add(_this3.componentNameDash + '-' + _this3.props.name);
			});
			return this;
		}

		/**
   * Close the drawer
   */

	}, {
		key: 'close',
		value: function close() {
			var _this4 = this;

			// uncheck the toggle
			this.mutate(function () {
				_this4.toggleElm.checked = false;
				_this4.toggleElm.removeAttribute('checked');
			});

			var transition = (0, _getTransitionProperties2.default)(this);
			setTimeout(function () {
				_this4.mutate(function () {
					document.body.classList.remove(_this4.componentNameDash + '-' + _this4.props.name);
				});
			}, transition.totalDuration);
			return this;
		}

		/**
   * Check if is opened
   * @return 		{Boolean} 		true if opened, false if not
   */

	}, {
		key: 'isOpen',
		value: function isOpen() {
			return this.toggleElm.checked;
		}
	}], [{
		key: 'defaultCss',


		/**
   * Css
   * @protected
   */
		value: function defaultCss(componentName, componentNameDash) {
			return '\n\t\t\t' + componentNameDash + ' {\n\t\t\t\tdisplay : block;\n\t\t\t}\n\t\t';
		}
	}, {
		key: 'defaultProps',


		/**
   * Default props
   * @definition 		SWebComponent.defaultProps
   * @protected
   */
		get: function get() {
			return {
				/**
     * Specify the name of the drawer to be able to access it later through the API
     * @prop
     * @type 		{String}
     */
				name: null,

				/**
     * Close the drawer when click on a link inside it
     * @prop
     * @type 		{Boolean}
     */
				closeOnClick: true,

				/**
     * Specify if need to check the hash to automatically open the drawer if match with the name
     * @prop
     * @type 		{Boolean}
     */
				handleHash: true,

				/**
     * Prevent the content from scrolling when the drawer is opened.
     * This will override your transitions on the content so be aware of that...
     * @prop
     * @type 	{Boolean}
     */
				preventContentScroll: false
			};
		}

		/**
   * Physical props
   * @definition 		SWebComponent.physicalProps
   * @protected
   */

	}, {
		key: 'physicalProps',
		get: function get() {
			return ['name'];
		}
	}]);

	return SDrawerComponent;
}(_SWebComponent3.default);

exports.default = SDrawerComponent;